

from django.shortcuts import render, get_object_or_404
from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.views import generic
#from django.http import Http404
from django.utils import timezone

from .models import Question,Choice

##from django.template import loader
#def index(request):
#    latest_question_list = Question.objects.order_by('-pub_date')[:5]
##    template = loader.get_template('polls/index.html')
#    context = {
#            'latest_question_list': latest_question_list,
#            }
##   return HttpResponse(template.render(context, request))
#    return render(request,'polls/index.html',context)
#
#    #output = ','.join([q.question_text for q in latest_question_list])
##    return HttpResponse(output)
#    #return HttpResponse("Hello, world. You're at the polls index.")
#
#def detail(request, question_id):
#    #try:
#    #    question = Question.objects.get(pk=question_id)
#    #except Question.DoesNotExist:
#    #    raise Http404("Question does not exist")
#    question = get_object_or_404(Question,pk=question_id)
#    return render(request, 'polls/detail.html', {'question': question})
#    #return HttpResponse("You're looking at question %s." % question_id )
#
#def results(request, question_id):
#    question = get_object_or_404(Question,pk=question_id)
#    return render(request, 'polls/results.html',{'question':question })
#    #response = "Your're loinging at the results of question %s."
#    #return HttpResponse( response % question_id )
class IndexView(generic.ListView):
    template_name = 'polls/index.html'
    context_object_name = 'latest_question_list'
#    queryset = Question.objects.order_by('-pub_date')[:5]

    def get_queryset(self):
        """Return the last five published questions """
    #    return Question.objects.order_by('-pub_date')[:5]
        return Question.objects.filter(
                pub_date__lte=timezone.now()
        ).order_by('-pub_date')[:5]


class DetailView(generic.DetailView):
    """
       get pk value from queryset (default: $Model.objects.all()) to context value in template。
       question = get_object_or_404(Question,pk=question_id)
       return render(request,"polls/detail.html",{'question':question})
    """
    model = Question
    template_name = 'polls/detail.html'

    def get_queryset(self):
        return Question.objects.filter(pub_date__lte=timezone.now())

class ResultsView(generic.DetailView):
    model = Question
    template_name = 'polls/results.html'

    def get_queryset(self):
        return Question.objects.filter(pub_date__lte=timezone.now())

def vote(request, question_id):
#    return HttpResponse("Your're voting on question %s." % question_id)
    question = get_object_or_404(Question, pk=question_id)
    try:
        selected_choice = question.choice_set.get(pk=request.POST['choice'])
    except (KeyError, Choice.DoesNotExist):
        return render(request, 'polls/detail.html', {
                'question': question,
                'error_message': "You didn't select a choice.",
            })
    else:
        selected_choice.votes += 1
        selected_choice.save()
        return HttpResponseRedirect(reverse('polls:results', args=(question.id,)))





from django.conf.urls import url

from . import views

app_name = 'polls'
#urlpatterns = [
#    url(r'^$', views.index, name='index'),
#    url(r'^(?P<question_id>[0-9]+)/$', views.detail, name='detail'),
#    #url(r'^specifics/(?P<question_id>[0-9]+)/$', views.detail, name='detail'),
#    url(r'(?P<question_id>[0-9]+)/results/$', views.results, name='results'),
#    url(r'(?P<question_id>[0-9]+)/vote/$', views.vote, name='vote'),
#
#]
urlpatterns = [
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^(?P<pk>[0-9]+)/$', views.DetailView.as_view(), name='detail'),
    url(r'^(?P<pk>[0-9]+)/results/$', views.ResultsView.as_view(), name='results'),
    url(r'^(?P<question_id>[0-9]+)/vote/$', views.vote, name='vote'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-18 06:53
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('polls', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='choice',
            old_name='choice_test',
            new_name='choice_text',
        ),
    ]



from django.contrib import admin

from .models import Question, Choice

# Register your models here.
class ChoiceInline(admin.TabularInline):
    model = Choice
    extra = 3

class QuestionAdmin(admin.ModelAdmin):
    #fields = [ 'pub_date', 'question_text' ]
    fieldsets = [
            (None, {'fields': ['question_text']}),
            ('Date Information', {'fields': ['pub_date']}),
            ]
    inlines = [ChoiceInline]
    list_display = ('question_text', 'pub_date', 'was_published_recently')
    list_filter = ['pub_date']
    search_fields = ['question_text']

admin.site.register(Question, QuestionAdmin)
admin.site.register(Choice)



from django.db import models
from django.utils import timezone
import datetime
# Create your models here.

class Question(models.Model):
    question_text = models.CharField(max_length=200)
    pub_date = models.DateTimeField('date published')

    def __str__(self):
        return self.question_text
    def was_published_recently(self):
        now = timezone.now()
        return  now - datetime.timedelta(days=1) <= self.pub_date <= now
    was_published_recently.admin_order_field = 'pub_date'
    was_published_recently.boolean = True
    was_published_recently.short_description = 'Published recently?'

class Choice(models.Model):
    question = models.ForeignKey(Question, on_delete=models.CASCADE)
    choice_text = models.CharField(max_length=200)
    votes = models.IntegerField(default=0)

    def __str__(self):
        return self.choice_text



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-18 03:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Choice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('choice_test', models.CharField(max_length=200)),
                ('votes', models.IntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question_text', models.CharField(max_length=200)),
                ('pub_date', models.DateTimeField(verbose_name='date published')),
            ],
        ),
        migrations.AddField(
            model_name='choice',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='polls.Question'),
        ),
    ]




# Create your tests here.

import datetime

from django.utils import timezone
from django.test import TestCase
from django.core.urlresolvers import reverse

from .models import Question

class QuestionMethodTests(TestCase):

    def test_was_published_recently_with_future_question(self):
        """
        was_published_recently() should return False for question whose
        pub_date is in the futrue.
        """
        time = timezone.now() + datetime.timedelta(days=30)
        future_question = Question(pub_date=time)
        self.assertEqual(future_question.was_published_recently(), False)

    def test_was_published_recently_with_old_question(self):
        """
        was_published_recently() should return False for question whose
        pub_date is in the futrue
        """
        time = timezone.now() -  datetime.timedelta(days=30)
        old_question = Question(pub_date=time)
        self.assertEqual(old_question.was_published_recently(), False)

    def test_was_published_recently_with_recent_question(self):
        """
        was_published_recently() should return True for question whose
        pub_date is in the future
        """
        time = timezone.now() -  datetime.timedelta(hours=1)
        recent_question = Question(pub_date=time)
        self.assertEqual(recent_question.was_published_recently(), True)

def create_question(question_text, days):
    """
    Creates a question with the given `question_text` and published the
    given number of `days` offset to now (negative for questions published
    in the past, positive for questions that have yet to be published).
    """
    time = timezone.now() + datetime.timedelta(days=days)
    return Question.objects.create(question_text=question_text, pub_date=time)


class QuestionViewTests(TestCase):
    def test_index_view_with_no_questions(self):
        """
        If no questions exist, an appropriate message should be displayed.
        """
        response = self.client.get(reverse('polls:index'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "No polls are available.")
        self.assertQuerysetEqual(response.context['latest_question_list'], [])

    def test_index_view_with_a_past_question(self):
        """
        Questions with a pub_date in the past should be displayed on the
        index page.
        """
        create_question(question_text="Past question.", days=-30)
        response = self.client.get(reverse('polls:index'))
        self.assertQuerysetEqual(
            response.context['latest_question_list'],
            ['<Question: Past question.>']
        )

    def test_index_view_with_a_future_question(self):
        """
        Questions with a pub_date in the future should not be displayed on
        the index page.
        """
        create_question(question_text="Future question.", days=30)
        response = self.client.get(reverse('polls:index'))
        self.assertContains(response, "No polls are available.")
        self.assertQuerysetEqual(response.context['latest_question_list'], [])

    def test_index_view_with_future_question_and_past_question(self):
        """
        Even if both past and future questions exist, only past questions
        should be displayed.
        """
        create_question(question_text="Past question.", days=-30)
        create_question(question_text="Future question.", days=30)
        response = self.client.get(reverse('polls:index'))
        self.assertQuerysetEqual(
            response.context['latest_question_list'],
            ['<Question: Past question.>']
        )

    def test_index_view_with_two_past_questions(self):
        """
        The questions index page may display multiple questions.
        """
        create_question(question_text="Past question 1.", days=-5)
        create_question(question_text="Past question 2.", days=-30)
        response = self.client.get(reverse('polls:index'))
        self.assertQuerysetEqual(
            response.context['latest_question_list'],
            ['<Question: Past question 1.>', '<Question: Past question 2.>']
        )

class QuestionIndexDetailTests(TestCase):
    def test_detail_view_with_a_future_question(self):
        """
        The detail view of a question with a pub_date in the future should
        return a 404 not found
        """
        future_question = create_question(question_text='Future question.',
                                        days=5)
        response = self.client.get(reverse('polls:detail',args=(future_question.id,)))
        self.assertEqual(response.status_code, 404)

    def test_detail_view_with_a_past_question(self):
        """
        The detail view of a question with a pub_date in the past should
        display the question's text
        """
        past_question = create_question(question_text='Past question.',
                                        days=-5)
        response = self.client.get(reverse('polls:detail',args=(past_question.id,)))
        self.assertContains(response,past_question.question_text, status_code=200)

class QuestionIndexResultsTests(TestCase):
    def test_results_view_with_a_future_question(self):
        """
        The reuslt view of a question with a pub_date in the future should
        return a 404 not found
        """
        future_question = create_question(question_text='Future question', days=30)
        response = self.client.get(reverse('polls:results',args=(future_question.id,)))
        self.assertEqual(response.status_code,404)

    def test_results_view_with_a_past_question(self):
        """
        The reuslt view of a question with a pub_date in the future should
        return a 404 not found
        """
        past_question = create_question(question_text='Future question', days=-30)
        response = self.client.get(reverse('polls:results',args=(past_question.id,)))
        self.assertContains(response,past_question.question_text,status_code=200)
